from TDhelper.generic.dictHelper import findInDict, hitDict,setDictValue,destoryKey,appendDict
class cache:
    __cache__={}

    @classmethod
    def insert(self,k,v):
        appendDict(k,v,self.__cache__)
    
    @classmethod
    def append(self,k,v):
        if k in self.__cache__:
            raise Exception("%s already."%k)
        appendDict(k,v,self.__cache__)

    @classmethod
    def update(self,k,v):
        setDictValue(k,v,self.__cache__)

    @classmethod
    def destory(self,k):
        destoryKey(k,self.__cache__)

    @classmethod
    def findValue(self,k):
        return findInDict(k,self.__cache__)

    @classmethod
    def findInstance(self,k):
        return hitDict(k,self.__cache__)

    @classmethod
    def __get__(self):
        return self.__cache__
    
    @classmethod
    def __items__(self):
        return self.__cache__.items()
    
    @classmethod
    def __clear__(self):
        self.__cache__.clear()


from TDhelper.Msg.Config import *
from TDhelper.apiCore import *
import time
import abc
import six


@six.add_metaclass(abc.ABCMeta)
class Msg():
    '''发送信息接口
       
       Args:
       config (class<MsgConfig>): Open api config.
       sendBy (str): send person, default value is "".
    '''
    @abc.abstractclassmethod
    def __init__(self, config:Config, sendBy:str=""):
        self._config=config
        self._sendBy=sendBy
        self._sendHandle=apiCore(self._config.PlatformName,config.Version)

    @abc.abstractclassmethod
    def Send(self, sendTo:str="", sendBuff:str="", apiKey:str="send"):
        '''立即发送

           Args:
           sendTo (str): message recive person, default value {}.  
           sendBuff (str): send content, default value "". 
           apiKey (str): send api keyword,default value is "send".
        '''
    @abc.abstractclassmethod
    def MultipleSend(self,sendTo:list=[], sendBuff:str="",apiKey:str="multiplesend"):
        '''批量发送

            Args:
           sendTo (array): message recive person, default value [].  
           sendBuff (str): send content, default value "". 
           apiKey (str): multiple send api keyword,default value is "multiplesend".
        '''

    @abc.abstractclassmethod
    def Install(self):
        '''You can add the api'''

    @abc.abstractclassmethod
    def Uninstall(self):
        '''You can delete all msg api'''



import urllib.request
import io
import os
import json
from TDhelper.bin.globalvar import *
from TDhelper.network.http.REST_HTTP import GET, POST


class apiCore():
    '''apiCore

       Args:
            apiName (str):Open Api platform's Name.
            version (str):Api's version, default value is "v1".
            apiRouteFileName (str):Location cache file's name, default value is "apiRoutePath"
            basePath (str):Location cache base path, defalut value is os.path.dirname(__file__)
    '''

    def __init__(self, apiName: str, version="v1", apiRouteFileName="apiRoutePath.arg", basePath=os.path.dirname(__file__)):
        self._apiName = apiName+"_"+version
        self._path = basePath+"\\"+apiName+"\\"+version+"\\"
        # if haven't this directory,create it.
        if not os.path.exists(self._path):
            os.makedirs(self._path)
        # generate the api route path
        self._routePath = os.path.join(self._path, apiRouteFileName)
        if not hasKey(apiName+version):
            # create global route tables
            setGlobalVariable(self._apiName, dict())
        if self._routePath:
            if os.path.exists(self._routePath):
                # read apiRouteFile and init apiroute
                try:
                    with open(self._routePath, mode='r', encoding='utf-8', errors=None, newline=None) as f:
                        while True:
                            line = f.readline().replace("\n", "")
                            if line:
                                jsondoc = json.loads(line)
                                if jsondoc["key"]:
                                    if jsondoc["value"]:
                                        self.AddApi(jsondoc["key"], jsondoc["value"]["uri"], jsondoc["value"]
                                                    ["args"], jsondoc["value"]["method"], jsondoc["value"]["descrition"])
                            else:
                                break
                        f.close()
                except Exception as e:
                    raise e

    def AddApi(self, key: str, apiUri: str, argsStr: str, method: str = "GET", descrition: str = ""):
        '''Add a new api

            Args:
                key (str): Api key.
                apiUri (str): Api uri.
                argsStr (str): Api uri args string format, Example: "gid={0}&name={1}".
                method (str): Http method, default value is "GET".
                descrition (str): api descrition.
        '''
        # if key.lower() not in self._apiRoute:
        if key.lower() not in getGlobalVariable(self._apiName):
            getGlobalVariable(self._apiName)[key.lower()] = {
                "uri": apiUri, "args": argsStr, "method": method.lower(), "descrition": descrition}
            return True
        else:
            return False

    def GetArgsString(self, key: str):
        '''Get api uri's param string

            Args:
                key (str): api key
        '''
        try:
            key = key.lower()
            if key in getGlobalVariable(self._apiName):
                if "args" in getGlobalVariable(self._apiName)[key]:
                    return getGlobalVariable(self._apiName)[key]["args"]
        except Exception as e:
            raise e

    def DelApi(self, key: str):
        '''Delete a api

            Args:
                key (str): api key
        '''
        key = key.lower()
        if key in getGlobalVariable(self._apiName):
            del getGlobalVariable(self._apiName)[key]
            return True
        else:
            return False

    def SaveToFile(self):
        '''Save api route to file
        '''
        wBuffer = ""  # create file write buffer
        for item in getGlobalVariable(self._apiName):
            wBuffer += json.dumps({"key": item,
                                   "value": getGlobalVariable(self._apiName)[item]})+"\r\n"
        if not wBuffer:
            wBuffer = "\r\n"
        if wBuffer:  # save buffer to file
            with open(self._routePath, mode='w', encoding='utf-8', errors=None, newline=None) as f:
                f.write(wBuffer)
                f.flush()
                f.close()

    def ClearCache(self):
        deleteGlobalVariable(self._apiName)  # 删除全局变量
        os.remove(self._routePath)  # 删除缓存文件
        os.removedirs(self._path)  # 删除缓存目录

    def Call(self, key: str, data: str):
        '''Call remote api

            Args:
                key (str): api key.
                data (str): api uri's params string.
        '''
        key = key.lower()
        if key in getGlobalVariable(self._apiName):
            method = getGlobalVariable(self._apiName)[key]["method"]
            uri = getGlobalVariable(self._apiName)[key]["uri"]
            if uri:
                if method.lower() == "get":
                    uri += "?"+data
                    return self._restfulGET(uri)
                elif method.low() == "post":
                    return self._restfulPOST(uri, data)
            return False
        return False
    def _restfulGET(self, uri):
        try:
            return GET(uri)
        except Exception as e:
            raise e

    def _restfulPOST(self, uri, data):
        try:
            return POST(uri, data)
        except Exception as e:
            raise e



#!/usr/bin/env python3.6
# -*- encoding: utf-8 -*-
"""
@File    :   recursion.py
@Time    :   2020/01/16 23:06:51
@Author  :   Tang Jing 
@Version :   1.0.0
@Contact :   yeihizhi@163.com
@License :   (C)Copyright 2020
@Desc    :   None
"""

# here put the import lib
import functools

# code start

def recursion(func):
    def wapper(*args, **kwargs):
        if kwargs["count"] == kwargs["upper-limit"]:
            return args, kwargs
        kwargs["count"] += 1
        args, kwargs = func(*args, **kwargs)      
        return args, kwargs
    return wapper

def recursionCall(func, upper_limit: int = 200, *args, **kwargs):
    assert upper_limit > 0, 'upper_limit is the call layer limit, must set greater than 0.'
    if "count" not in kwargs:
        kwargs["count"] = 1
    if "limit" not in kwargs:
        kwargs['limit']= upper_limit
    if "upper-limit" not in kwargs:
        kwargs["upper-limit"] = upper_limit
    if "break" not in kwargs:
        kwargs["break"] = True
    while kwargs["break"]:
        args, kwargs = func(*args, **kwargs)
        kwargs["upper-limit"] = kwargs["count"] + upper_limit
    return args, kwargs